/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.email.inboxlistener.engine;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.log4j.Logger;

/**
 * Provides a db connection pool using a
 * <code>org.apache.commons.dbcp.BasicDataSource</code>.
 * It's a singleton class but it must be configured calling the method
 * <code>configure(BasicDataSource)</code>
 * @version $Id: DBConnectionProvider.java,v 1.2 2007/01/15 16:37:39 nichele Exp $
 */
public class DBConnectionProvider {

    // ------------------------------------------------------------ Private data

    /**
     * The used logger
     */
    private static final Logger logger = Logger.getLogger("funambol.pushlistener.dbconnectionprovider");

    /**
     * The datasource
     */
    private static BasicDataSource basicDataSource = null;

    /**
     * The DBConnectionProvider instance
     */
    private static DBConnectionProvider instance = new DBConnectionProvider();

    // ------------------------------------------------------------ Constructors

    /**
     * Just to implement the singleton pattern
     */
    private DBConnectionProvider() {

    }

    // ---------------------------------------------------------- Public methods

    /**
     * Returns an available connection
     * @throws java.sql.SQLException if an error occurs
     * @return an available connection
     */
    public static Connection getConnection() throws SQLException {
        return instance.getConn();
    }

    /**
     * Closes the datasource
     * @throws java.sql.SQLException if an error occurs
     */
    public static void close() throws SQLException {
        basicDataSource.close();
    }

    /**
     * Returns the max active cnnections of the DBConnectionProvider
     * @return the max active cnnections of the DBConnectionProvider
     */
    public static int getMaxActive() {
        return basicDataSource.getNumActive();
    }
    
    /**
     * Returns the max idle cnnections of the DBConnectionProvider
     * @return the max idle cnnections of the DBConnectionProvider
     */
    public static int getMaxIdle() {
        return basicDataSource.getMaxIdle();
    }

    // ------------------------------------------------------- Protected methods
    /**
     * Configures the DBConnectionProvider instance. This is protected so just the class
     * in this package (like the PushListener) can configure it.
     * @param ds the BasicDataSource
     * @throws java.sql.SQLException if an error occurs
     */
    protected static void configure(BasicDataSource ds) throws SQLException {
        if (logger.isInfoEnabled()) {
            logger.info("Configuring DBConnection provider with: " + basicDataSource);
        }
        if (ds == null) {
            return ;
        }
        if (basicDataSource != null) {
            if (logger.isInfoEnabled()) {
                logger.info("There is an already configured BasicDatasource....closing it");
            }
            basicDataSource.close();
        }
        basicDataSource = ds;
    }

    // --------------------------------------------------------- Private methods
    /**
     * Returns an available connection
     * @throws java.sql.SQLException if an error occurs
     * @return an available connection
     */
    private Connection getConn() throws SQLException {
        return basicDataSource.getConnection();
    }

}
