/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;

import java.io.IOException;
import java.net.SocketException;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

/**
 * Simple tools to work with the MBean exposed by the inboxlistener.
 * It provides a way to stop the inboxlistener calling "stop" operation.
 *
 * @version $Id: ManagementTools.java,v 1.4 2007/03/03 12:12:22 nichele Exp $
 */
public class ManagementTools {

    // --------------------------------------------------------------- Constants
    /** Command to use to stop the InboxListener */
    private static final String COMMAND_STOP   = "stop";

    /** Command to see the InboxListener status */
    private static final String COMMAND_STATUS = "Status";

    /** Operation to call to stop the Inboxlistener */
    private static final String OPERATION_STOP   = "stop";

    // ------------------------------------------------------------ Private data

    /** MBeanServerConnection */
    private MBeanServerConnection mbsc       = null;

    /** ObjectName to work with the mbean Inboxlistener */
    private ObjectName objectName = null;

    /** Host name */
    private String host = null;

    /** Port */
    private int port = 0;

    /** Username */
    private String userName = null;

    /** Password */
    private String password = null;

    // ------------------------------------------------------------ Constructors

    /**
     * Creates a new ManagementTools
     * @param host the host name
     * @param port the port number
     * @param userName the user name
     * @param password the password
     * @throws java.lang.Exception if an error occurs
     */
    public ManagementTools(String host, int port, String userName, String password)
    throws Exception {

        this.host     = host;
        this.port     = port;
        this.userName = userName;
        this.password = password;

        objectName = new ObjectName(MessageProcessorMBean.MBEAN_NAME);
        mbsc       = getMBeanServerConnection();
    }

    /**
     * Runs ManagementTools
     * @param args arguments
     */
    public static void main(String[] args) {
        int numArgs = args.length;
        if (args.length > 0) {
            if (args[0].equals("-h")    ||
                args[0].equals("-help") ||
                args[0].equals("-?")    ||
                args[0].equals("/?")     ) {

                usage();
                return;
            }
        }

        if (numArgs != 3 && numArgs != 5) {
                usage();
                return;
        }

        String host  = args[0];
        String sPort = args[1];
        int    port  = 0;
        try {
            port = Integer.parseInt(sPort);
        } catch (NumberFormatException e) {
            System.out.println("Port number not valid.");
            System.exit(-1);
        }

        String command  = args[numArgs - 1];
        String password = null;
        String userName = null;
        if (numArgs == 5) {
            password = args[numArgs - 2];
            userName = args[numArgs - 3];
        }

        ManagementTools tools = null;
        try {
            tools = new ManagementTools(host, port, userName, password);
        } catch (IOException ex) {
            System.out.println("Unable to connect to " + host + ":" + port);
            System.exit(-1);
        } catch (Exception ex) {
            System.err.println("Error creating ManagementTools");
            ex.printStackTrace(System.err);
            System.exit(-1);
        }

        if (COMMAND_STOP.equalsIgnoreCase(command)) {
            tools.stop();
        } else if (COMMAND_STATUS.equalsIgnoreCase(command)) {
            //
            // Showing always this message because if the inboxlistener is not
            // running (or it is running on a different port) an exception is 
            // caught previously 
            //
            System.out.println("InboxListener status: running");
        }
    }

    /**
     * Shows ManagementTools usage
     */
    private static void usage() {
        System.err.println("Usage: java com.funambol.email.inboxlistener.engine.ManagementTools " +
                           "host port [username] [password] stop|status");
    }

    /**
     * Stops the InboxListener calling "stop" operation
     */
    private void stop() {
        try {
            mbsc.invoke(objectName, OPERATION_STOP, null, null);
        } catch (java.rmi.UnmarshalException ue) {
            Throwable t = ue.getCause();
            if (t instanceof SocketException ||
                t instanceof java.io.EOFException) {
                // this means the inboxlistener shut down
            } else {
                ue.printStackTrace(System.err);
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    /**
     * Returns a MBeanServerConnection
     * @throws java.lang.Exception if an error occurs
     * @return MBeanServerConnection
     */
    private MBeanServerConnection getMBeanServerConnection()
    throws Exception {
        //
        // Create an RMI connector client and connect it to the RMI connector server
        //
        System.out.println("Connecting to: " + host + ":" + port);

        final String urlPath = "/jndi/rmi://" + host + ":" + port + "/jmxrmi";
        JMXServiceURL url = new JMXServiceURL("rmi", "", 0, urlPath);

        Map<String, String[]> env = new HashMap<String, String[]>();
        env.put(JMXConnector.CREDENTIALS,
                new String[] {userName, password});
        JMXConnector jmxc = JMXConnectorFactory.connect(url, env);
        //
        // Get an MBeanServerConnection
        //
        MBeanServerConnection mbsc = jmxc.getMBeanServerConnection();
        return mbsc;
    }
}