/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.security;

import java.io.Serializable;

import java.security.Principal;

import com.funambol.framework.core.Authentication;
import com.funambol.framework.core.Cred;
import com.funambol.framework.filter.WhereClause;
import com.funambol.framework.security.AbstractOfficer;
import com.funambol.framework.security.Officer;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Sync4jUser;
import com.funambol.framework.server.inventory.DeviceInventory;
import com.funambol.framework.server.inventory.DeviceInventoryException;
import com.funambol.framework.server.store.NotFoundException;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.framework.tools.Base64;

import com.funambol.server.admin.UserManager;
import com.funambol.server.config.Configuration;

import com.funambol.email.console.dao.ConsoleDAO;
import com.funambol.email.engine.source.EmailConnectorConfig;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.util.TokenUtility;
import com.funambol.framework.server.Sync4jDevice;

/**
 * This is an implementation of the <i>Officier</i> interface.
 * It always authenticates and authorizes users and resource accesses.
 *
 * @version $Id: EmailOfficer.java,v 1.29 2007/03/23 11:16:11 gbmiglia Exp $
 */
public class EmailOfficer
extends AbstractOfficer
implements Officer, Serializable {

    // ---------------------------------------------------------- Protected data
    protected PersistentStore ps          = null;
    protected UserManager     userManager = null;

    // -------------------------------------------------------------- Properties
    private String jndiDataSourceName = null;
    private String encryptionKey      = null;

    // ------------------------------------------------------------ Constructors
    public EmailOfficer() {
        super();
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Gets the supported authentication type
     *
     * @return the basic authentication type
     */
    public String getClientAuth() {
        return Cred.AUTH_TYPE_BASIC;
    }

    /**
     * Authenticates a credential.
     *
     * @param credential the credential to be authenticated
     *
     * @return the Sync4jUser if the credential is autenticated, null otherwise
     */
    public Sync4jUser authenticateUser(Cred credential) {
        Configuration config = Configuration.getConfiguration();
        userManager = config.getUserManager();
        ps = config.getStore();
        String type = credential.getType();
        if ((Cred.AUTH_TYPE_BASIC).equals(type)) {
            return authenticateBasicCredential(credential);
        }
        return null;
    }

    // --------------------------------------------------------- Private methods

    /**
     * Checks the given B64 credential.
     *
     * @param credential the cred to check
     *
     * @return the Sync4jUser if the credential is autenticated, null otherwise
     */
    private Sync4jUser authenticateBasicCredential(Cred credential) {

        String username, password;

        Authentication auth = credential.getAuthentication();
        String deviceId = auth.getDeviceId();
        String userpwd = new String(Base64.decode(auth.getData()));

        int p = userpwd.indexOf(':');

        if (p == -1) {
            username = userpwd;
            password = "";
        } else {
            username = (p > 0) ? userpwd.substring(0, p) : "";
            password = (p == (userpwd.length() - 1)) ? "" : userpwd.substring(p + 1);
        }

        if (log.isTraceEnabled()) {
            log.trace("Username: " + username);
        }

        try {
            this.jndiDataSourceName =
                    EmailConnectorConfig.getConfigInstance().getDataSource();
        } catch (EmailConfigException e) {
            log.error("Error Getting Connector Parameters ", e);
        }


        this.encryptionKey = TokenUtility.getToken();

        Sync4jUser dsuser = null;

        try {

            // check user in the DS-Server Schema
            dsuser = existsUser(username);

            if (dsuser != null) {

                // user exists in the ds-schema
                if (log.isTraceEnabled()) {
                    log.trace("User with '" + username + "' exists.");
                }

                if (existsDevice(deviceId)){

                    // device exists in the ds-schema
                    if (log.isTraceEnabled()) {
                        log.trace("Device with '" + deviceId + "' exists.");
                    }

                    // insert principal if it doesn't exists
                    if (!existsPrincipal(username, deviceId)) {
                        long pri = insertPrincipal(username, deviceId);
                        credential.getAuthentication().setPrincipalId(pri);
                    }

                    // Verify these credentials on Mail Server
                    if (!checkCredentials(username, password)) {
                        if (log.isTraceEnabled()) {
                            log.trace("User with '" + username +
                                    "' exists, but has an invalid Mail Server Account.");
                        }
                        return null;
                    }

                }

            }

        } catch (PersistentStoreException e) {
            log.error("Error inserting a new user / principal: ", e);
            return null;
        } catch (Exception e) {
            log.error("Error checking the user / principal: ", e);
            return null;
        }

        return dsuser;

    }


    /**
     *
     * Checks if there is an user with the given username
     *
     * @param userName the username
     * @return true if the user exists, false otherwise
     */
    private boolean checkCredentials(String userName, String password)
     throws Exception {

        boolean  check = false;
        boolean  error = false;

        try {

            ConsoleDAO cdao = new ConsoleDAO(this.jndiDataSourceName,
                                             this.encryptionKey);

            MailServerAccount msa = cdao.getUser(userName);

            if (msa != null){

                String MSlogin            = msa.getMsLogin();
                String MSPassword         = msa.getMsPassword();
                String host               = msa.getMailServer().getInServer();
                int    port               = msa.getMailServer().getInPort();
                String protocol           = msa.getMailServer().getProtocol();
                boolean isSSLIn           = msa.getMailServer().getIsSSLIn();
                String keystorePath       = msa.getMailServer().getKeystorePath();
                String keystorePassphrase = msa.getMailServer().getKeystorePwd();

                try {
                    IMailServerWrapper mswf =
                             MailServerWrapperFactory.getMailServerWrapper(protocol);

                     mswf.checkUser(host, "" + port, MSlogin, MSPassword,
                                    isSSLIn, keystorePath, keystorePassphrase);

                } catch (Exception e){
                     log.error("Officer Error; No connection for user: " + userName);
                     error = true;
                }

                if (!error) {
                    check = true;
                }

            }

        } catch (Exception e) {
            throw new Exception("Error checking username in the Email Schema" + e.getMessage());
        }

        return check;
    }

    /**
     * Checks if there is an user with the given username
     *
     * @param userName the username
     * @return true if the user exists, false otherwise
     */
    private Sync4jUser existsUser(String userName) throws PersistentStoreException {

        Sync4jUser[] users;
        WhereClause wc;

        String value[] = new String[]{userName};
        wc = new WhereClause("username", value, WhereClause.OPT_EQ, true);
        users = userManager.getUsers(wc);

        Sync4jUser user = null;
        if (users.length > 0){
            user = users[0];
        }

        return user;

    }

    /**
     * Checks if there is an user with the given username
     *
     * @param userName the username
     * @return true if the user exists, false otherwise
     */
    private boolean existsDevice(String deviceId) throws DeviceInventoryException {

        boolean exists = false;

        DeviceInventory devInventory =
                Configuration.getConfiguration().getDeviceInventory();

        exists = devInventory.getDevice(new Sync4jDevice(deviceId), false);
        
        return exists;
    }

    /**
     * Verify if there is a principal with the given userName and deviceId in store
     *
     * @param userName the username
     * @param deviceId the device's id
     * @throws PersistentStoreException if an error occurs
     */
    private boolean existsPrincipal(String userName, String deviceId)
    throws PersistentStoreException {
        Principal principal;
        try {
            principal = Sync4jPrincipal.createPrincipal(userName, deviceId);
            ps.read(principal);
            return true;
        } catch (NotFoundException e) {
            if (log.isTraceEnabled()) {
                log.trace("Principal for " + userName + ":" + deviceId + " not found");
            }
            return false;
        }
    }

    /**
     * Insert a principal
     *
     * @param userName the username
     * @param deviceId the device's id
     * @return the id of the principal
     * @throws PersistentStoreException if an error occurs
     */
    private long insertPrincipal(String userName, String deviceId)
    throws PersistentStoreException {
        Sync4jPrincipal principal =
            Sync4jPrincipal.createPrincipal(userName, deviceId);
        try {
            ps.store(principal);
        } catch (PersistentStoreException e) {
            log.error("Error creating new principal: ", e);
            if (log.isTraceEnabled()) {
                log.trace("authenticateBasicCredential", e);
            }
            throw e;
        }
        return principal.getId();
    }

}
